#!/usr/bin/env python3

from distutils.core import setup, Extension
import glob
import os
from DistUtilsExtra.command import *

setup(name='system-service',
      version='0.3',
      packages = ['UbuntuSystemService',
                 ],
      data_files=[
                  # dbus stuff
                  ('share/dbus-1/system-services',
                   ['backend/com.ubuntu.SystemService.service']),
                  ('../etc/dbus-1/system.d/',
                   ["backend/com.ubuntu.SystemService.conf"]),
                  ('lib/system-service/',
                   ["backend/system-service-d"]),
                  ],
      cmdclass = { "build" : build_extra.build_extra,
                   "build_i18n" :  build_i18n.build_i18n,
                   "build_help" :  build_help.build_help,
                   "build_icons" :  build_icons.build_icons }
      )
